// LVSpeechPort_Streaming_ExampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "LVSpeechPort_Streaming_Example.h"
#include "LVSpeechPort_Streaming_ExampleDlg.h"
#include "About.h"
#include "AudioVolume.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// The speech channel we're using
#define SPEECH_VOICE_CHANNEL	0

// Use up to the last 10 volume readings to calculate average
#define MAX_AVERAGE_VOLUME		10

#define GRAMMAR_LABEL "StreamingExampleGrammar"


/////////////////////////////////////////////////////////////////////////////
// CSpeechPortDlg dialog

CSpeechPortDlg::CSpeechPortDlg(CWnd* pParent /*=NULL*/)
: CDialog(CSpeechPortDlg::IDD, pParent)
, m_MicrophoneOn(FALSE)
, m_StreamActive(false)
, m_Running(true)
, m_StreamStatus(-1)
, m_MicrophoneActive(false)
, m_VolumeMeterUpdateCounter(0)
{
	//{{AFX_DATA_INIT(CSpeechPortDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSpeechPortDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSpeechPortDlg)
	DDX_Check(pDX, IDC_CHECK_MIC_ACTIVE, m_MicrophoneOn);
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CSpeechPortDlg, CDialog)
	//{{AFX_MSG_MAP(CSpeechPortDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_CHECK_MIC_ACTIVE, OnCheckMicActive)
	ON_CBN_DBLCLK(IDC_GRAMMAR, OnGrammarListDoubleClick)
	ON_LBN_DBLCLK(IDC_GRAMMAR, OnGrammarListDoubleClick)
	ON_BN_CLICKED(IDC_EXIT, OnExit)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_CLEAR, &CSpeechPortDlg::OnClearClicked)
	ON_MESSAGE(MSG_START_STREAMING, OnStartStreaming)
END_MESSAGE_MAP()




/////////////////////////////////////////////////////////////////////////////
// CSpeechPortDlg message handlers

BOOL CSpeechPortDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// Add Grammar Strings to the list control (based on contents of grammar file)
	SendDlgItemMessage(IDC_GRAMMAR, LB_ADDSTRING, 0, (LPARAM) "YES");
	SendDlgItemMessage(IDC_GRAMMAR, LB_ADDSTRING, 0, (LPARAM) "NO");
	SendDlgItemMessage(IDC_GRAMMAR, LB_ADDSTRING, 0, (LPARAM) "YELLOW");
	SendDlgItemMessage(IDC_GRAMMAR, LB_ADDSTRING, 0, (LPARAM) "BLUE");
	SendDlgItemMessage(IDC_GRAMMAR, LB_ADDSTRING, 0, (LPARAM) "WHITE");

	// Send message to self to begin streaming
	PostMessage(MSG_START_STREAMING, 0L, 0L);

	return TRUE;  // return TRUE  unless you set the focus to a control
}


// Handles begin streaming message
//
LRESULT CSpeechPortDlg::OnStartStreaming(WPARAM wParam, LPARAM lParam)
{

	m_StreamActive				= false;
	m_MicrophoneActive			= false;

	m_wfx.wFormatTag			= WAVE_FORMAT_PCM;
	m_wfx.nChannels				= 1;
	m_wfx.nSamplesPerSec		= 16000;
	m_wfx.nAvgBytesPerSec		= 32000;
	m_wfx.nBlockAlign			= 2;
	m_wfx.wBitsPerSample		= 16;
	m_wfx.cbSize				= 0;

	m_wfx.wFormatTag			= WAVE_FORMAT_MULAW;
	m_wfx.nChannels				= 1;
	m_wfx.nSamplesPerSec		= 8000;
	m_wfx.nAvgBytesPerSec		= 8000;
	m_wfx.nBlockAlign			= 1;
	m_wfx.wBitsPerSample		= 8;
	m_wfx.cbSize				= 0;

	m_StreamStatus				= -1;

	CWnd* pWorkingLabel = GetDlgItem(IDC_WORKING);
	if (pWorkingLabel) 
		pWorkingLabel->ShowWindow(SW_HIDE);

	CWnd* pRecordingLabel = GetDlgItem(IDC_RECORDING);
	if (pRecordingLabel)
		pRecordingLabel->ShowWindow(SW_HIDE);

	if(StartRecord() == false)
		MessageBox("Problem starting wave stream", "Start Streaming", MB_ICONEXCLAMATION | MB_OK);

	// Open the speech port
	m_Port.OpenPort();

	m_Port.SetPropertyEx(PROP_SAVE_SOUND_FILES, PROP_EX_VALUE_TYPE_INT, (void*)1,PROP_EX_TARGET_CLIENT, 0);	//save SRE tuner log files to disk
	m_Port.AddEvent(EVENT_START_DECODE_SEQ);		//lets SRE know to start a new tuner file

	// Starts the worker thread, which waits for SRE to finish a decode
	AfxBeginThread(ThreadProc, (LPVOID)this);

	// Tell the audio stream to begin accepting audio
	StartStream(true); 

	return 0;
}


void CSpeechPortDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}


// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSpeechPortDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}


HCURSOR CSpeechPortDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}


void CSpeechPortDlg::OnCheckMicActive() 
{
	UpdateData(true);
	
	if(m_MicrophoneOn)
		m_MicrophoneActive = true;
	else
		m_MicrophoneActive = false;
}


BOOL CSpeechPortDlg::DestroyWindow() 
{
	// Signal the worker thread to stop
	m_Running = false;

	// Wait up to 1 second for worker to stop
	WaitForSingleObject(m_WorkerThreadFinished, 1000);

	// Release memory in m_VolumeList
	POSITION Pos = m_VolumeList.GetHeadPosition();
	CAudioVolume *pAudioVolume;

	while(Pos)
	{
		pAudioVolume = (CAudioVolume *)m_VolumeList.GetNext(Pos);
		delete pAudioVolume;
	}

	// Close the speech port
	m_Port.ClosePort();

	// Perform cleanup and flush any callsre file(s) before the speech interface is release
	LV_SRE_Shutdown();

	return CDialog::DestroyWindow();
}


// The enter key maps to hidden OK button
void CSpeechPortDlg::OnOK() 
{
	char buffer[1024];
	SendDlgItemMessage(IDC_ADD_GRAMMAR, WM_GETTEXT, 1024, (LPARAM) buffer);	// Get new string of the edit control
	SendDlgItemMessage(IDC_ADD_GRAMMAR, WM_SETTEXT, 0, (LPARAM) "");		// Clear edit control ready for new string

	if (strlen(buffer) < 1)
		return;

	strupr(buffer);
	if (SendDlgItemMessage(IDC_GRAMMAR, LB_FINDSTRINGEXACT, 0, (LPARAM) buffer) == LB_ERR)
	{
		SendDlgItemMessage(IDC_GRAMMAR, LB_ADDSTRING, 0, (LPARAM) buffer);
		RefreshGrammar();
	}
}


// Called when user double-clicks on the list (deleting the selected entry)
//
void CSpeechPortDlg::OnGrammarListDoubleClick() 
{
	SendDlgItemMessage(IDC_GRAMMAR, LB_DELETESTRING, SendDlgItemMessage(IDC_GRAMMAR, LB_GETCURSEL, 0, 0), 0);
	RefreshGrammar();
}


void CSpeechPortDlg::OnExit() 
{
	// Close the application
	CDialog::OnOK();
}


// Called when user wants to clear the output window
//
void CSpeechPortDlg::OnClearClicked()
{
	// Clear the SRE list
	SendDlgItemMessage(IDC_ANSWER_LIST, LB_RESETCONTENT, 0, 0);
}


// The main worker thread routine (processes audio and speech decodes)
//
UINT CSpeechPortDlg::ThreadProc(LPVOID pParam)
{
	CString OutputString;

	CSpeechPortDlg * pParentDialog = (CSpeechPortDlg *)pParam;

	CWnd* pWorkingLabel = pParentDialog->GetDlgItem(IDC_WORKING);


	while (pParentDialog->m_Running) // While the parent dialog tells us to keep running
	{
		Sleep(100);

		if(WaitForSingleObject(pParentDialog->m_DecodeStartedEvent, 0) == WAIT_OBJECT_0)
		{
			// Decode has started (end of speech was detected in the callback)

			// Show the 'working' message in the dialog
			if (pWorkingLabel != NULL) 
				pWorkingLabel->ShowWindow(SW_SHOW);

			// Get the number of concepts detected
			// RR 04.28.2010 Support for Concept-Phrase grammar is being dropped
			// so switching to semantic interpretation 
			int AnswerCount = pParentDialog->m_Port.GetNumberOfInterpretations(0);

			// Get the number of items in the answer list so that we can append
			long ListCount =  (long)pParentDialog->SendDlgItemMessage(IDC_ANSWER_LIST, LB_GETCOUNT, 0, 0);

			TRACE("Interpretations Detected: %d\n", AnswerCount);

			// Cycle through all detected concepts and add each to the answer list
			for (int i = 0 ; i < AnswerCount ; i++)
			{
				// RR 04.28.2010 switching to semantic interpretation
				const char * GetDecodeSI = pParentDialog->m_Port.GetInterpretationString(0, i);
				int GetDecodeScore = pParentDialog->m_Port.GetInterpretationScore(0, i);
				OutputString.Format("%s  -  %i", GetDecodeSI, GetDecodeScore);//RR 08.01.2006 - changed GetRawTextDecoded to GetConcept this works better

				pParentDialog->SendDlgItemMessage(IDC_ANSWER_LIST, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR)OutputString);
				pParentDialog->SendDlgItemMessage(IDC_ANSWER_LIST, LB_SETCURSEL, ListCount + i, 0);  //to keep new items in view
			}

			// Restart the audio stream now that we're done processing
			pParentDialog->StartStream(false);

			// hide the 'working' message in the dialog
			if (pWorkingLabel != NULL) 
				pWorkingLabel->ShowWindow(SW_HIDE);
		}
	}

	// Signal parent dialog that the worker is finished
	pParentDialog->m_WorkerThreadFinished.SetEvent();

	return 0L;
}


// Start or restart the audio stream to allow new speech data to be received. This
// routine also sets the grammar that will be used on the audio data when decoding
// FirstTime indicates whether this is the first time this routine is called, allowing
// some initialization to be done.
//
long CSpeechPortDlg::StartStream(bool FirstTime)
{
	CSingleLock singleLock(&m_csStream);
	singleLock.Lock();

	if(FirstTime)
	{
		//set all parameters for the stream (only do this the first time through)

		m_Port.StreamSetParameter(STREAM_PARM_DETECT_BARGE_IN, 1);	//stream will detect barge-in (until barge in detected streamed data is discarded)
		m_Port.StreamSetParameter(STREAM_PARM_AUTO_DECODE, 1);		//when end-of speech is detected (or StopStream() called) recognition will begin immediately
		m_Port.StreamSetParameter(STREAM_PARM_DETECT_END_OF_SPEECH, 1);
		m_Port.StreamSetParameter(STREAM_PARM_BARGE_IN_DYNAMIC_ADJUST, 1);


		m_Port.StreamSetParameter(STREAM_PARM_VOICE_CHANNEL, 0);			//stream will use audio channel 0
		m_Port.StreamSetParameter(STREAM_PARM_GRAMMAR_SET, LV_ACTIVE_GRAMMAR_SET);	//stream will use active grammar set
		//	m_Port.StreamSetParameter(STREAM_PARM_SOUND_FORMAT, PCM_16KHZ);  //set audio format os stream
		m_Port.StreamSetParameter(STREAM_PARM_SOUND_FORMAT, ULAW_8KHZ);  //set audio format of stream

		//	m_Port.StreamSetParameter(STREAM_PARM_END_OF_SPEECH_DETECTION, STREAM_END_OF_SPEECH_DETECTION_SINGLE_WORDS); //adjusts how quickly end-of-speech is triggered

		//	m_Port.StreamSetParameter(STREAM_PARM_BARGE_IN_NOISE_COUNT_LOW_THRESHOLD, 10); //adjusts how quickly end-of-speech is triggered

		//	m_Port.StreamSetParameter(STREAM_PARM_NOISE_REDUCTION, 1); //adjusts how quickly end-of-speech is triggered

		//begin delay make it more difficult to barge-in at beginning of prompt
		//	this is useful when echo-canceling audio and you need to wait for convergence
		//in this case we're not playing a prompt so we don't want the delay
		//	m_Port.StreamSetParameter(STREAM_PARM_BARGE_IN_BEGIN_DELAY, 0);


		// Sets a call back for the state change info
		m_Port.StreamSetStateChangeCallBack(LVSpeechPortCallBack, this);

		RefreshGrammar();
	}

	// Start or restart the audio stream so that speech can be detected
	int RetVal = m_Port.StreamStart();

	if (RetVal != LV_SUCCESS) 
	{
		CString ErrorMessage;
		ErrorMessage.Format("Stream Start Error# %d", RetVal);
		MessageBox(ErrorMessage, "ERROR", MB_ICONEXCLAMATION | MB_OK);
		return -1;
	}

	m_StreamActive = true;

	return 0;
}


// static - Callback routine from the speech port, used to indicate when various speech
// events have taken place.
//
void CSpeechPortDlg::LVSpeechPortCallBack(int NewState, unsigned int TotalBytes, unsigned int RecordedBytes, void* UserData)
{
	CSpeechPortDlg* This = (CSpeechPortDlg*) UserData;

	if (NewState == STREAM_STATUS_BARGE_IN)
	{
		CWnd* RecordingLabel = This->GetDlgItem(IDC_RECORDING);
		if (RecordingLabel) 
			RecordingLabel->ShowWindow(SW_SHOW);
	}

	if (NewState == STREAM_STATUS_END_SPEECH)
	{
		CWnd* RecordingLabel = This->GetDlgItem(IDC_RECORDING);
		if (RecordingLabel) 
			RecordingLabel->ShowWindow(SW_HIDE);

		{
			CSingleLock singleLock(&This->m_csStream);
			singleLock.Lock();
			This->m_StreamActive = false;
			This->m_DecodeStartedEvent.SetEvent();
		}
	}

	if (NewState == STREAM_STATUS_STOPPED)
	{
		CWnd* RecordingLabel = This->GetDlgItem(IDC_RECORDING);
		if (RecordingLabel) 
			RecordingLabel->ShowWindow(SW_HIDE);

		{
			CSingleLock singleLock(&This->m_csStream);
			singleLock.Lock();
			This->m_StreamActive = false;
		}
	}
}


// Starts the wave audio streamer
//
bool CSpeechPortDlg::StartRecord()
{

	int PlayCount = 0;
	int RetVal = 0;

	while (1)
	{
		RetVal = waveInOpen(&m_hWaveIn, WAVE_MAPPER, &m_wfx, (DWORD_PTR) MMCallBack, (DWORD_PTR) this, CALLBACK_FUNCTION);
		PlayCount++;
		if (RetVal == 0) 
			break;
		Sleep(200);
		if (PlayCount > 10) 
			break;
	}


	if (IsError(RetVal, "StartRecord::waveInOpen"))
		return false;


	for (int i=0 ; i < NUM_IN_BUFFERS ; i++)
	{
		m_whdrIn[i].lpData				= m_audioBufIn[i];
		m_whdrIn[i].dwBufferLength		= IN_BUFFER_SIZE;
		m_whdrIn[i].dwBytesRecorded		= 0;
		m_whdrIn[i].dwUser				= 0;
		m_whdrIn[i].dwLoops = 0;
		m_whdrIn[i].dwFlags = 0;
		m_whdrIn[i].reserved = 0;
		m_whdrIn[i].lpNext = 0;

		if (IsError(waveInPrepareHeader(m_hWaveIn, &(m_whdrIn[i]), sizeof(WAVEHDR)), "StartRecord::waveInPrepareHeader)"))
			return false;
		if (IsError(waveInAddBuffer(m_hWaveIn, &(m_whdrIn[i]), sizeof(WAVEHDR)), "StartRecord::waveInAddBuffer)"))
			return false;
	}


	if (IsError(waveInStart(m_hWaveIn), "StartRecord::waveInStart"))
		return false;

	return true;
}


//static
void CALLBACK CSpeechPortDlg::MMCallBack(HWAVEIN hwi,UINT Msg, DWORD Instance, DWORD lParam, DWORD rParam)
{
	CSpeechPortDlg* This = (CSpeechPortDlg*) Instance;

	switch(Msg)
	{

	case WIM_OPEN:
		break;

	case WIM_CLOSE:
		break;

	case WIM_DATA:
		This->StreamIn((WAVEHDR*) lParam);
		break;
	}
}


static short const muLawToPCM[256] = {
	-32124,		-31100,		-30076,		-29052,		-28028,		-27004,		-25980,		-24956,
	-23932,		-22908,		-21884,		-20860,		-19836,		-18812,		-17788,		-16764,
	-15996,		-15484,		-14972,		-14460,		-13948,		-13436,		-12924,		-12412,
	-11900,		-11388,		-10876,		-10364,		-9852,		-9340,		-8828,		-8316,
	-7932,		-7676,		-7420,		-7164,		-6908,		-6652,		-6396,		-6140,
	-5884,		-5628,		-5372,		-5116,		-4860,		-4604,		-4348,		-4092,
	-3900,		-3772,		-3644,		-3516,		-3388,		-3260,		-3132,		-3004,
	-2876,		-2748,		-2620,		-2492,		-2364,		-2236,		-2108,		-1980,
	-1884,		-1820,		-1756,		-1692,		-1628,		-1564,		-1500,		-1436,
	-1372,		-1308,		-1244,		-1180,		-1116,		-1052,		-988,		-924,
	-876,		-844,		-812,		-780,		-748,		-716,		-684,		-652,
	-620,		-588,		-556,		-524,		-492,		-460,		-428,		-396,
	-372,		-356,		-340,		-324,		-308,		-292,		-276,		-260,
	-244,		-228,		-212,		-196,		-180,		-164,		-148,		-132,
	-120,		-112,		-104,		-96,		-88,		-80,		-72,		-64,
	-56,		-48,		-40,		-32,		-24,		-16,		-8,			0,
	32124,		31100,		30076,		29052,		28028,		27004,		25980,		24956,
	23932,		22908,		21884,		20860,		19836,		18812,		17788,		16764,
	15996,		15484,		14972,		14460,		13948,		13436,		12924,		12412,
	11900,		11388,		10876,		10364,		9852,		9340,		8828,		8316,
	7932,		7676,		7420,		7164,		6908,		6652,		6396,		6140,
	5884,		5628,		5372,		5116,		4860,		4604,		4348,		4092,
	3900,		3772,		3644,		3516,		3388,		3260,		3132,		3004,
	2876,		2748,		2620,		2492,		2364,		2236,		2108,		1980,
	1884,		1820,		1756,		1692,		1628,		1564,		1500,		1436,
	1372,		1308,		1244,		1180,		1116,		1052,		988,		924,
	876,		844,		812,		780,		748,		716,		684,		652,
	620,		588,		556,		524,		492,		460,		428,		396,
	372,		356,		340,		324,		308,		292,		276,		260,
	244,		228,		212,		196,		180,		164,		148,		132,
	120,		112,		104,		96,			88,			80,			72,			64,
	56,			48,			40,			32,			24,			16,			8,			0
};


long CSpeechPortDlg::StreamIn(WAVEHDR* wavehdr)
{
	if (m_Running == false)
		return 0;

	//{
	//	// Optionally Save the streamed audio to disk
	//	FILE* fp = fopen("C:\\streaming.pcm", "ab");

	//	if(fp != NULL)
	//	{
	//		fwrite(wavehdr->lpData, wavehdr->dwBytesRecorded, 1, fp);
	//		fclose(fp);
	//	}
	//}


	if(m_MicrophoneActive)
	{
		{
			unsigned long Sum = 0;
			unsigned char* s = (unsigned char*) wavehdr->lpData;
			unsigned long Length = wavehdr->dwBytesRecorded;
			long Sample;

			for (unsigned long i = 0 ; i < Length  ; i++)
			{
				Sample = muLawToPCM[s[i]];
				if (Sample > 0)
					Sum += Sample;
				else
					Sum += (-Sample);
			}

			AddVolume(Sum / Length);

			if ((++m_VolumeMeterUpdateCounter % 3) == 0)	//update display every 3rd time
			{
				unsigned long AverageVolume = (unsigned long)(GetAverageVolume() * 0.1);
				SendDlgItemMessage(IDC_MIC_LEVEL, PBM_SETPOS, AverageVolume, 0);
			}
		}

		{
			CSingleLock singleLock(&m_csStream);
			singleLock.Lock();

			if (m_StreamActive)
			{
				m_Port.StreamSendData(wavehdr->lpData, wavehdr->dwBytesRecorded);
				long NewStreamStatus =  m_Port.StreamGetStatus();

				if (m_StreamStatus != NewStreamStatus)
				{
					CString MessageString;

					static char *StreamStates[] = {
						"Not Ready",			//	0 must call LV_SRE_StreamStart function to set up new stream
						"Ready",				//	1 port can accept streamed data
						"Barge In",				//	2 barge-in detected, now saving streamed data (user code can stop playback of audio data)
						"End Speech",			//	3 when returned , new stream data no longer accepted until one of the StartStream functions called
						"Status Stopped",		//	4 StreamStop was called, 
						"Barge In Timeout",		//  5 barge-in not triggered before timeout, no audio will be saved.
						"End Speech Timeout",	//	6 end-speech not detected.  call StreamStop to force a decode if in AutoDecode mode.
					};

					if(NewStreamStatus >= 0 && NewStreamStatus <= (sizeof(StreamStates) / sizeof(StreamStates[0])))
					{					
						// State string information available					
						MessageString.Format("Status: %s (%d)", StreamStates[NewStreamStatus], NewStreamStatus);
					}
					else
					{					
						MessageString.Format("Status: %d", NewStreamStatus);
					}

					SendDlgItemMessage(IDC_ANSWER_LIST, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR)MessageString);

					m_StreamStatus = NewStreamStatus;
				}
			}

		}
	}
	else
	{
		SendDlgItemMessage(IDC_MIC_LEVEL, PBM_SETPOS, 0, 0);
	}

	IsError(waveInAddBuffer(m_hWaveIn, wavehdr, sizeof(WAVEHDR)), "StreamIn::waveInAddBuffer");

	//PBM_SETPOS
	return 0;
}


// Record the current volume so that this can be later used to get an average
//
void CSpeechPortDlg::AddVolume(unsigned int NewVolume)
{
	if(m_VolumeList.GetSize() > MAX_AVERAGE_VOLUME)
	{
		CAudioVolume *pAudioVolume = (CAudioVolume *)m_VolumeList.RemoveHead();
		delete pAudioVolume;
	}

	m_VolumeList.AddTail(new CAudioVolume(NewVolume));
}


// Returns the average volume
//
unsigned int CSpeechPortDlg::GetAverageVolume()
{
	int NumEntries = (int)m_VolumeList.GetSize();

	if(NumEntries < 1)
		return 0;

	// Iterate through the list to count the total of all volumes
	unsigned int Total = 0;
	POSITION Pos = m_VolumeList.GetHeadPosition();
	CAudioVolume *pAudioVolume;

	while(Pos)
	{
		pAudioVolume = (CAudioVolume *)m_VolumeList.GetNext(Pos);
		Total += pAudioVolume->m_Volume;
	}

	// Now divide by number of volume entries to calculate average volume to be returned
	return Total / NumEntries;
}



const char * GetAudioErrorString( MMRESULT r )
{
	switch(r)
	{
	case MMSYSERR_NOERROR:
		return "" ;
	case MMSYSERR_ALLOCATED:
		return  ("Specified resource is already allocated.");
	case MMSYSERR_BADDEVICEID:
		return  ("Specified device identifier is out of range.");
	case MMSYSERR_NODRIVER:
		return  ("No device driver is present.");
	case MMSYSERR_NOMEM:
		return  ("Unable to allocate or lock memory.");
	case WAVERR_BADFORMAT:
		return  ("Attempted to open with an unsupported waveform-audio format.");
	case MMSYSERR_INVALHANDLE:
		return  ("Specified device handle is invalid.");
	case WAVERR_STILLPLAYING:
		return  ("The buffer pointed to by the LPWAVEHDR parameter is still in the queue.");
	case WAVERR_UNPREPARED:
		return  ("The buffer pointed to by the LPWAVEHDR parameter hasn't been prepared.");
	case MMSYSERR_NOTSUPPORTED:
		return  ("Function is not supported.");
	case MMSYSERR_INVALPARAM:
		return  ("Invalid parameter.");
	}
	return ("Unknown Error");
}


bool CSpeechPortDlg::IsError(MMRESULT err, const char* FunctionMsg)
{
	if (err == MMSYSERR_NOERROR) 
		return false;

	MessageBox(GetAudioErrorString(err), "Audio Error", MB_ICONWARNING | MB_OK);
	return true;
}


// RR 04.29.2010 AddPhrasesToGrammar is a function that will take the phrases
// given to it (from the list) and dynamically build a grammar based on those phrases.  This
// is a simple grammar, for more complex examples refer to: 
// http://www.lumenvox.com/help/speechEngine/grammars/writingGrammars/srgs-introduction.htm
void CSpeechPortDlg::AddPhrasesToGrammar(const char *GrammarString)
{

	char *GrammarHeader = "#ABNF 1.0 UTF-8;\r\nlanguage en-us;\r\nmode voice;\r\ntag-format <semantics/1.0.2006>;\r\nroot $LVRootRule;\r\npublic $LVRootRule = \r\n";

	// First add the header (that specifies the grammar type and format, etc.)
	m_GrammarBufferString = GrammarHeader;

	// Then the grammar string made up from the list of words
	m_GrammarBufferString += GrammarString;

	// m_GrammarBufferString now contains the complete new grammar, and this will be loaded
	// as a buffer shortly (in the RefreshGrammar routine)

}


// Rebuilds and reloads the grammar based on the current list
//
void CSpeechPortDlg::RefreshGrammar()
{
	//
	// (Re)Populate the grammar based on the contents of the list box
	//

	// First, we need to deactivate any previous grammar that may still be active
	m_Port.DeactivateGrammars();

	char buffer[1024];

	// Get the number of words currently in the list
	int NumItemsInList = (int)SendDlgItemMessage(IDC_GRAMMAR, LB_GETCOUNT, 0, 0);

	CString GrammarString;

	// Cycle through all items in the list
	for (int Row = 0 ; Row < NumItemsInList ; Row++)
	{

		// RR 04.28.2010 Switching to SRGS Grammars
		// Now we collect all the phrases and pass it to the port via LoadGrammar

		// Get the specified message from the list box
		SendDlgItemMessage(IDC_GRAMMAR, LB_GETTEXT, Row, (LPARAM) buffer);

		if(Row < NumItemsInList-1)
		{
			// RR 04.29.2010 separate each phrase with a single OR pipe...
			GrammarString += buffer;
			GrammarString += " |\r\n";
		}
		else
		{
			// RR 04.29.2010 ...except the last phrase which concludes the grammar
			// with the semi-colon.
			GrammarString += buffer;
			GrammarString += ";\r\n";
		}
	}

	// RR 04.29.2010 Pass the phrases to the function that will dynamically populate m_GrammarBufferString that is loaded below
	AddPhrasesToGrammar(GrammarString);


	// RR 04.29.2010 After the grammar has been created in the dynamic buffer (m_GrammarBufferString) it is loaded here.
	int LoadGrammarResult = m_Port.LoadGrammarFromBuffer(GRAMMAR_LABEL, (LPCTSTR)m_GrammarBufferString);

	if(LoadGrammarResult != LV_SUCCESS)
	{
		CString ErrorMessage;
		ErrorMessage.Format("Error code %d when loading grammar (%s)", LoadGrammarResult, LV_SRE_ReturnErrorString(LoadGrammarResult));
		MessageBox(ErrorMessage, "ERROR", MB_ICONEXCLAMATION | MB_OK);
	}
	else
	{
		// Activate the new grammar (referenced by its label)
		int ActivateGrammarResult = m_Port.ActivateGrammar(GRAMMAR_LABEL);

		if(ActivateGrammarResult != LV_SUCCESS)
		{
			CString ErrorMessage;
			ErrorMessage.Format("Error code %d when activating grammar (%s)", ActivateGrammarResult, LV_SRE_ReturnErrorString(ActivateGrammarResult));
			MessageBox(ErrorMessage, "ERROR", MB_ICONEXCLAMATION | MB_OK);
		}
	}

}

