// LVSpeechPort_Streaming_ExampleDlg.h : header file
//

#if !defined(AFX_LVSPEECHPORT_STREAMING_EXAMPLEDLG_H__98EF1C1E_ED20_481B_AE01_F4323A1B1DAC__INCLUDED_)
#define AFX_LVSPEECHPORT_STREAMING_EXAMPLEDLG_H__98EF1C1E_ED20_481B_AE01_F4323A1B1DAC__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <LVSpeechPort.h>


#define IN_BUFFER_SIZE 1024
#define NUM_IN_BUFFERS 6


#define MSG_START_STREAMING		(WM_USER+101)



/////////////////////////////////////////////////////////////////////////////
// CSpeechPortDlg dialog

class CSpeechPortDlg : public CDialog
{
// Construction
public:
	CSpeechPortDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	//{{AFX_DATA(CSpeechPortDlg)
	enum { IDD = IDD_LVSPEECHPORT_STREAMING_EXAMPLE_DIALOG };
	BOOL				m_MicrophoneOn;
		// NOTE: the ClassWizard will add data members here
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSpeechPortDlg)
public:
	virtual BOOL DestroyWindow();

protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

	// Triggering when end-of-speech state reached in SRE, tells thread to begin waiting for decode to end
	CEvent				m_DecodeStartedEvent;

	// Triggered by worker thread to indicate it is finished (after receiving m_Running == false notification)
	CEvent				m_WorkerThreadFinished;

	// The speech port object
	LVSpeechPort		m_Port;

	// Flag indicating whether audio stream to speech port is active
	volatile bool		m_StreamActive;

	// Used to signal the worker thread when we're shutting down
	volatile bool		m_Running;

	// Handle to incoming wave stream
	HWAVEIN				m_hWaveIn;

	// Wave audio format
	WAVEFORMATEX		m_wfx;

	// Wave audio input buffers
	WAVEHDR				m_whdrIn[NUM_IN_BUFFERS];

	// Audio buffers
	CHAR				m_audioBufIn[NUM_IN_BUFFERS][IN_BUFFER_SIZE];	

	// Status of the speech port stream
	long				m_StreamStatus;

	// Flag indicating whether the microphone is active
	volatile bool		m_MicrophoneActive;

	// Used to periodically update the volume display
	unsigned int		m_VolumeMeterUpdateCounter;

	// Keeps track of the past few volume measurements
	CObList				m_VolumeList;

	// Critical section to lock access to the stream
	CCriticalSection	m_csStream;

	// Holds the grammar we dynamically create in AddPhrasesToGrammar
	CString				m_GrammarBufferString;

// Implementation
protected:
	HICON				m_hIcon;

	// The main worker thread routine
	static UINT ThreadProc(LPVOID pParam);

	// Starts or restarts the audio stream to allow new speech data to be received
	long StartStream(bool FirstTime);

	// Starts the wave audio streamer
	bool StartRecord();

	// Receives audio from the wave streaming interface
	long StreamIn(WAVEHDR* wavehdr);

	// Records the current volume so that this can be later used to get an average
	void AddVolume(unsigned int NewVolume);

	// Returns the average volume
	unsigned int GetAverageVolume();

	// Rebuilds and reloads the grammar based on the current list
	void RefreshGrammar();

	static void LVSpeechPortCallBack(int NewState, unsigned int TotalBytes, unsigned int RecordedBytes, void* UserData);
	static void CALLBACK MMCallBack(HWAVEIN hwi,UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);

	bool IsError(MMRESULT err, const char* FunctionMsg);

	// Create a dynamic grammar buffer from a string of phrases (from the list)
	void AddPhrasesToGrammar(const char *GrammarString);

	// Generated message map functions
	//{{AFX_MSG(CSpeechPortDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnCheckMicActive();
	virtual void OnOK();
	afx_msg void OnGrammarListDoubleClick();
	afx_msg void OnExit();
	//}}AFX_MSG
	afx_msg LRESULT OnStartStreaming(WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnClearClicked();
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_LVSPEECHPORT_STREAMING_EXAMPLEDLG_H__98EF1C1E_ED20_481B_AE01_F4323A1B1DAC__INCLUDED_)
